/****************************************************************************
** Meta object code from reading C++ file 'collectionscanner.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../musique/src/collectionscanner.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionscanner.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CollectionScanner_t {
    QByteArrayData data[23];
    char stringdata0[280];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CollectionScanner_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CollectionScanner_t qt_meta_stringdata_CollectionScanner = {
    {
QT_MOC_LITERAL(0, 0, 17), // "CollectionScanner"
QT_MOC_LITERAL(1, 18, 8), // "progress"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 8), // "finished"
QT_MOC_LITERAL(4, 37, 5), // "stats"
QT_MOC_LITERAL(5, 43, 5), // "error"
QT_MOC_LITERAL(6, 49, 7), // "message"
QT_MOC_LITERAL(7, 57, 13), // "scanDirectory"
QT_MOC_LITERAL(8, 71, 4), // "QDir"
QT_MOC_LITERAL(9, 76, 9), // "directory"
QT_MOC_LITERAL(10, 86, 12), // "popFromQueue"
QT_MOC_LITERAL(11, 99, 20), // "giveThisFileAnArtist"
QT_MOC_LITERAL(12, 120, 9), // "FileInfo*"
QT_MOC_LITERAL(13, 130, 4), // "file"
QT_MOC_LITERAL(14, 135, 13), // "processArtist"
QT_MOC_LITERAL(15, 149, 13), // "gotArtistInfo"
QT_MOC_LITERAL(16, 163, 25), // "giveThisFileAnAlbumArtist"
QT_MOC_LITERAL(17, 189, 18), // "processAlbumArtist"
QT_MOC_LITERAL(18, 208, 19), // "giveThisFileAnAlbum"
QT_MOC_LITERAL(19, 228, 12), // "processAlbum"
QT_MOC_LITERAL(20, 241, 12), // "gotAlbumInfo"
QT_MOC_LITERAL(21, 254, 12), // "processTrack"
QT_MOC_LITERAL(22, 267, 12) // "emitFinished"

    },
    "CollectionScanner\0progress\0\0finished\0"
    "stats\0error\0message\0scanDirectory\0"
    "QDir\0directory\0popFromQueue\0"
    "giveThisFileAnArtist\0FileInfo*\0file\0"
    "processArtist\0gotArtistInfo\0"
    "giveThisFileAnAlbumArtist\0processAlbumArtist\0"
    "giveThisFileAnAlbum\0processAlbum\0"
    "gotAlbumInfo\0processTrack\0emitFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CollectionScanner[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       3,    1,   92,    2, 0x06 /* Public */,
       5,    1,   95,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,   98,    2, 0x08 /* Private */,
      10,    0,  101,    2, 0x08 /* Private */,
      11,    1,  102,    2, 0x08 /* Private */,
      14,    1,  105,    2, 0x08 /* Private */,
      15,    0,  108,    2, 0x08 /* Private */,
      16,    1,  109,    2, 0x08 /* Private */,
      17,    1,  112,    2, 0x08 /* Private */,
      18,    1,  115,    2, 0x08 /* Private */,
      19,    1,  118,    2, 0x08 /* Private */,
      20,    0,  121,    2, 0x08 /* Private */,
      21,    1,  122,    2, 0x08 /* Private */,
      22,    0,  125,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QVariantMap,    4,
    QMetaType::Void, QMetaType::QString,    6,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,

       0        // eod
};

void CollectionScanner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CollectionScanner *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->progress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->finished((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->error((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->scanDirectory((*reinterpret_cast< const QDir(*)>(_a[1]))); break;
        case 4: _t->popFromQueue(); break;
        case 5: _t->giveThisFileAnArtist((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 6: _t->processArtist((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 7: _t->gotArtistInfo(); break;
        case 8: _t->giveThisFileAnAlbumArtist((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 9: _t->processAlbumArtist((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 10: _t->giveThisFileAnAlbum((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 11: _t->processAlbum((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 12: _t->gotAlbumInfo(); break;
        case 13: _t->processTrack((*reinterpret_cast< FileInfo*(*)>(_a[1]))); break;
        case 14: _t->emitFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CollectionScanner::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionScanner::progress)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CollectionScanner::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionScanner::finished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CollectionScanner::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionScanner::error)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CollectionScanner::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CollectionScanner.data,
    qt_meta_data_CollectionScanner,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CollectionScanner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CollectionScanner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CollectionScanner.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CollectionScanner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void CollectionScanner::progress(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CollectionScanner::finished(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CollectionScanner::error(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
