/* $BEGIN_LICENSE

This file is part of Musique.
Copyright 2013, Flavio Tordini <flavio.tordini@gmail.com>

Musique is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Musique is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Musique.  If not, see <http://www.gnu.org/licenses/>.

$END_LICENSE */

#ifndef IMAGEDOWNLOADER_H
#define IMAGEDOWNLOADER_H

#include <QtCore>

class ImageDownload;

class ImageDownloader : public QObject {
    Q_OBJECT

public:
    static ImageDownloader &instance();

    enum ImageDownloadTypes { ArtistType = 0, AlbumType };

    void enqueue(int objectId, int objectType, const QString &url);
    void start();

public slots:
    void onImageDownloaded(const QByteArray &bytes);
    void imageDownloadError();

signals:
    void progress(int);
    void error(QString message);
    void finished();

private slots:
    void popFromQueue();

private:
    ImageDownloader(QObject *parent = nullptr);

    ImageDownload *imageDownload = nullptr;

    bool running = false;
};

#endif // IMAGEDOWNLOADER_H
