/****************************************************************************
** Meta object code from reading C++ file 'playlistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../musique/src/playlistmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PlaylistModel_t {
    QByteArrayData data[21];
    char stringdata0[224];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlaylistModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlaylistModel_t qt_meta_stringdata_PlaylistModel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PlaylistModel"
QT_MOC_LITERAL(1, 14, 16), // "activeRowChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 3), // "row"
QT_MOC_LITERAL(4, 36, 6), // "manual"
QT_MOC_LITERAL(5, 43, 13), // "startPlayback"
QT_MOC_LITERAL(6, 57, 16), // "needSelectionFor"
QT_MOC_LITERAL(7, 74, 15), // "QVector<Track*>"
QT_MOC_LITERAL(8, 90, 11), // "itemChanged"
QT_MOC_LITERAL(9, 102, 5), // "total"
QT_MOC_LITERAL(10, 108, 16), // "playlistFinished"
QT_MOC_LITERAL(11, 125, 8), // "addTrack"
QT_MOC_LITERAL(12, 134, 6), // "Track*"
QT_MOC_LITERAL(13, 141, 5), // "track"
QT_MOC_LITERAL(14, 147, 9), // "addTracks"
QT_MOC_LITERAL(15, 157, 9), // "newTracks"
QT_MOC_LITERAL(16, 167, 5), // "clear"
QT_MOC_LITERAL(17, 173, 12), // "skipBackward"
QT_MOC_LITERAL(18, 186, 11), // "skipForward"
QT_MOC_LITERAL(19, 198, 12), // "getNextTrack"
QT_MOC_LITERAL(20, 211, 12) // "trackRemoved"

    },
    "PlaylistModel\0activeRowChanged\0\0row\0"
    "manual\0startPlayback\0needSelectionFor\0"
    "QVector<Track*>\0itemChanged\0total\0"
    "playlistFinished\0addTrack\0Track*\0track\0"
    "addTracks\0newTracks\0clear\0skipBackward\0"
    "skipForward\0getNextTrack\0trackRemoved"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlaylistModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   69,    2, 0x06 /* Public */,
       6,    1,   76,    2, 0x06 /* Public */,
       8,    1,   79,    2, 0x06 /* Public */,
      10,    0,   82,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    1,   83,    2, 0x0a /* Public */,
      14,    1,   86,    2, 0x0a /* Public */,
      16,    0,   89,    2, 0x0a /* Public */,
      17,    0,   90,    2, 0x0a /* Public */,
      18,    0,   91,    2, 0x0a /* Public */,
      19,    0,   92,    2, 0x0a /* Public */,
      20,    0,   93,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    3,    4,    5,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 7,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 12,
    QMetaType::Void,

       0        // eod
};

void PlaylistModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlaylistModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeRowChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 1: _t->needSelectionFor((*reinterpret_cast< QVector<Track*>(*)>(_a[1]))); break;
        case 2: _t->itemChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->playlistFinished(); break;
        case 4: _t->addTrack((*reinterpret_cast< Track*(*)>(_a[1]))); break;
        case 5: _t->addTracks((*reinterpret_cast< QVector<Track*>(*)>(_a[1]))); break;
        case 6: _t->clear(); break;
        case 7: _t->skipBackward(); break;
        case 8: _t->skipForward(); break;
        case 9: { Track* _r = _t->getNextTrack();
            if (_a[0]) *reinterpret_cast< Track**>(_a[0]) = std::move(_r); }  break;
        case 10: _t->trackRemoved(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<Track*> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Track* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<Track*> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlaylistModel::*)(int , bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::activeRowChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PlaylistModel::*)(QVector<Track*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::needSelectionFor)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PlaylistModel::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::itemChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PlaylistModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::playlistFinished)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PlaylistModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_PlaylistModel.data,
    qt_meta_data_PlaylistModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlaylistModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlaylistModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int PlaylistModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void PlaylistModel::activeRowChanged(int _t1, bool _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PlaylistModel::needSelectionFor(QVector<Track*> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PlaylistModel::itemChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void PlaylistModel::playlistFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
