/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../musique/src/mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[76];
    char stringdata0[973];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 18), // "currentTimeChanged"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 1), // "s"
QT_MOC_LITERAL(4, 33, 17), // "collectionCreated"
QT_MOC_LITERAL(5, 51, 13), // "showMediaView"
QT_MOC_LITERAL(6, 65, 10), // "transition"
QT_MOC_LITERAL(7, 76, 20), // "showChooseFolderView"
QT_MOC_LITERAL(8, 97, 20), // "toggleContextualView"
QT_MOC_LITERAL(9, 118, 18), // "hideContextualView"
QT_MOC_LITERAL(10, 137, 20), // "updateContextualView"
QT_MOC_LITERAL(11, 158, 6), // "Track*"
QT_MOC_LITERAL(12, 165, 5), // "track"
QT_MOC_LITERAL(13, 171, 15), // "showInitialView"
QT_MOC_LITERAL(14, 187, 4), // "quit"
QT_MOC_LITERAL(15, 192, 11), // "showMessage"
QT_MOC_LITERAL(16, 204, 7), // "message"
QT_MOC_LITERAL(17, 212, 11), // "handleError"
QT_MOC_LITERAL(18, 224, 7), // "restore"
QT_MOC_LITERAL(19, 232, 15), // "messageReceived"
QT_MOC_LITERAL(20, 248, 6), // "goBack"
QT_MOC_LITERAL(21, 255, 9), // "visitSite"
QT_MOC_LITERAL(22, 265, 8), // "lazyInit"
QT_MOC_LITERAL(23, 274, 6), // "donate"
QT_MOC_LITERAL(24, 281, 11), // "reportIssue"
QT_MOC_LITERAL(25, 293, 5), // "about"
QT_MOC_LITERAL(26, 299, 16), // "toggleFullscreen"
QT_MOC_LITERAL(27, 316, 21), // "updateUIForFullscreen"
QT_MOC_LITERAL(28, 338, 10), // "setShuffle"
QT_MOC_LITERAL(29, 349, 7), // "enabled"
QT_MOC_LITERAL(30, 357, 9), // "setRepeat"
QT_MOC_LITERAL(31, 367, 18), // "maybeShowUpdateNag"
QT_MOC_LITERAL(32, 386, 4), // "stop"
QT_MOC_LITERAL(33, 391, 12), // "stateChanged"
QT_MOC_LITERAL(34, 404, 12), // "Media::State"
QT_MOC_LITERAL(35, 417, 5), // "state"
QT_MOC_LITERAL(36, 423, 11), // "searchFocus"
QT_MOC_LITERAL(37, 435, 4), // "tick"
QT_MOC_LITERAL(38, 440, 4), // "time"
QT_MOC_LITERAL(39, 445, 8), // "volumeUp"
QT_MOC_LITERAL(40, 454, 10), // "volumeDown"
QT_MOC_LITERAL(41, 465, 16), // "toggleVolumeMute"
QT_MOC_LITERAL(42, 482, 13), // "volumeChanged"
QT_MOC_LITERAL(43, 496, 9), // "newVolume"
QT_MOC_LITERAL(44, 506, 18), // "volumeMutedChanged"
QT_MOC_LITERAL(45, 525, 5), // "muted"
QT_MOC_LITERAL(46, 531, 13), // "startFullScan"
QT_MOC_LITERAL(47, 545, 3), // "dir"
QT_MOC_LITERAL(48, 549, 16), // "fullScanFinished"
QT_MOC_LITERAL(49, 566, 5), // "stats"
QT_MOC_LITERAL(50, 572, 20), // "startIncrementalScan"
QT_MOC_LITERAL(51, 593, 23), // "incrementalScanProgress"
QT_MOC_LITERAL(52, 617, 7), // "percent"
QT_MOC_LITERAL(53, 625, 23), // "incrementalScanFinished"
QT_MOC_LITERAL(54, 649, 6), // "search"
QT_MOC_LITERAL(55, 656, 5), // "query"
QT_MOC_LITERAL(56, 662, 18), // "suggestionAccepted"
QT_MOC_LITERAL(57, 681, 11), // "Suggestion*"
QT_MOC_LITERAL(58, 693, 10), // "suggestion"
QT_MOC_LITERAL(59, 704, 13), // "searchCleared"
QT_MOC_LITERAL(60, 718, 21), // "showActionInStatusBar"
QT_MOC_LITERAL(61, 740, 8), // "QAction*"
QT_MOC_LITERAL(62, 749, 4), // "show"
QT_MOC_LITERAL(63, 754, 28), // "showStopAfterThisInStatusBar"
QT_MOC_LITERAL(64, 783, 16), // "toggleScrobbling"
QT_MOC_LITERAL(65, 800, 6), // "enable"
QT_MOC_LITERAL(66, 807, 16), // "enableScrobbling"
QT_MOC_LITERAL(67, 824, 17), // "disableScrobbling"
QT_MOC_LITERAL(68, 842, 12), // "lastFmLogout"
QT_MOC_LITERAL(69, 855, 12), // "savePlaylist"
QT_MOC_LITERAL(70, 868, 12), // "loadPlaylist"
QT_MOC_LITERAL(71, 881, 20), // "toggleMenuVisibility"
QT_MOC_LITERAL(72, 902, 31), // "toggleMenuVisibilityWithMessage"
QT_MOC_LITERAL(73, 934, 17), // "toggleToolbarMenu"
QT_MOC_LITERAL(74, 952, 11), // "runFinetune"
QT_MOC_LITERAL(75, 964, 8) // "filename"

    },
    "MainWindow\0currentTimeChanged\0\0s\0"
    "collectionCreated\0showMediaView\0"
    "transition\0showChooseFolderView\0"
    "toggleContextualView\0hideContextualView\0"
    "updateContextualView\0Track*\0track\0"
    "showInitialView\0quit\0showMessage\0"
    "message\0handleError\0restore\0messageReceived\0"
    "goBack\0visitSite\0lazyInit\0donate\0"
    "reportIssue\0about\0toggleFullscreen\0"
    "updateUIForFullscreen\0setShuffle\0"
    "enabled\0setRepeat\0maybeShowUpdateNag\0"
    "stop\0stateChanged\0Media::State\0state\0"
    "searchFocus\0tick\0time\0volumeUp\0"
    "volumeDown\0toggleVolumeMute\0volumeChanged\0"
    "newVolume\0volumeMutedChanged\0muted\0"
    "startFullScan\0dir\0fullScanFinished\0"
    "stats\0startIncrementalScan\0"
    "incrementalScanProgress\0percent\0"
    "incrementalScanFinished\0search\0query\0"
    "suggestionAccepted\0Suggestion*\0"
    "suggestion\0searchCleared\0showActionInStatusBar\0"
    "QAction*\0show\0showStopAfterThisInStatusBar\0"
    "toggleScrobbling\0enable\0enableScrobbling\0"
    "disableScrobbling\0lastFmLogout\0"
    "savePlaylist\0loadPlaylist\0"
    "toggleMenuVisibility\0"
    "toggleMenuVisibilityWithMessage\0"
    "toggleToolbarMenu\0runFinetune\0filename"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      57,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  299,    2, 0x06 /* Public */,
       4,    0,  302,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,  303,    2, 0x0a /* Public */,
       5,    0,  306,    2, 0x2a /* Public | MethodCloned */,
       7,    1,  307,    2, 0x0a /* Public */,
       7,    0,  310,    2, 0x2a /* Public | MethodCloned */,
       8,    0,  311,    2, 0x0a /* Public */,
       9,    0,  312,    2, 0x0a /* Public */,
      10,    1,  313,    2, 0x0a /* Public */,
      13,    0,  316,    2, 0x0a /* Public */,
      14,    0,  317,    2, 0x0a /* Public */,
      15,    1,  318,    2, 0x0a /* Public */,
      17,    1,  321,    2, 0x0a /* Public */,
      18,    0,  324,    2, 0x0a /* Public */,
      19,    1,  325,    2, 0x0a /* Public */,
      20,    0,  328,    2, 0x0a /* Public */,
      21,    0,  329,    2, 0x0a /* Public */,
      22,    0,  330,    2, 0x08 /* Private */,
      23,    0,  331,    2, 0x08 /* Private */,
      24,    0,  332,    2, 0x08 /* Private */,
      25,    0,  333,    2, 0x08 /* Private */,
      26,    0,  334,    2, 0x08 /* Private */,
      27,    0,  335,    2, 0x08 /* Private */,
      28,    1,  336,    2, 0x08 /* Private */,
      30,    1,  339,    2, 0x08 /* Private */,
      31,    0,  342,    2, 0x08 /* Private */,
      32,    0,  343,    2, 0x08 /* Private */,
      33,    1,  344,    2, 0x08 /* Private */,
      36,    0,  347,    2, 0x08 /* Private */,
      37,    1,  348,    2, 0x08 /* Private */,
      39,    0,  351,    2, 0x08 /* Private */,
      40,    0,  352,    2, 0x08 /* Private */,
      41,    0,  353,    2, 0x08 /* Private */,
      42,    1,  354,    2, 0x08 /* Private */,
      44,    1,  357,    2, 0x08 /* Private */,
      46,    1,  360,    2, 0x08 /* Private */,
      48,    1,  363,    2, 0x08 /* Private */,
      50,    0,  366,    2, 0x08 /* Private */,
      51,    1,  367,    2, 0x08 /* Private */,
      53,    1,  370,    2, 0x08 /* Private */,
      54,    1,  373,    2, 0x08 /* Private */,
      56,    1,  376,    2, 0x08 /* Private */,
      59,    0,  379,    2, 0x08 /* Private */,
      60,    2,  380,    2, 0x08 /* Private */,
      63,    1,  385,    2, 0x08 /* Private */,
      64,    1,  388,    2, 0x08 /* Private */,
      66,    0,  391,    2, 0x08 /* Private */,
      67,    0,  392,    2, 0x08 /* Private */,
      68,    0,  393,    2, 0x08 /* Private */,
      69,    0,  394,    2, 0x08 /* Private */,
      70,    0,  395,    2, 0x08 /* Private */,
      71,    0,  396,    2, 0x08 /* Private */,
      72,    0,  397,    2, 0x08 /* Private */,
      73,    0,  398,    2, 0x08 /* Private */,
      74,    0,  399,    2, 0x08 /* Private */,
      74,    1,  400,    2, 0x08 /* Private */,
      74,    1,  403,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 34,   35,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,   38,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,   43,
    QMetaType::Void, QMetaType::Bool,   45,
    QMetaType::Void, QMetaType::QString,   47,
    QMetaType::Void, QMetaType::QVariantMap,   49,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   52,
    QMetaType::Void, QMetaType::QVariantMap,   49,
    QMetaType::Void, QMetaType::QString,   55,
    QMetaType::Void, 0x80000000 | 57,   58,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 61, QMetaType::Bool,    2,   62,
    QMetaType::Void, QMetaType::Bool,   62,
    QMetaType::Void, QMetaType::Bool,   65,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QVariantMap,   49,
    QMetaType::Void, QMetaType::QString,   75,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentTimeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->collectionCreated(); break;
        case 2: _t->showMediaView((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->showMediaView(); break;
        case 4: _t->showChooseFolderView((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->showChooseFolderView(); break;
        case 6: _t->toggleContextualView(); break;
        case 7: _t->hideContextualView(); break;
        case 8: _t->updateContextualView((*reinterpret_cast< Track*(*)>(_a[1]))); break;
        case 9: _t->showInitialView(); break;
        case 10: _t->quit(); break;
        case 11: _t->showMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->handleError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->restore(); break;
        case 14: _t->messageReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->goBack(); break;
        case 16: _t->visitSite(); break;
        case 17: _t->lazyInit(); break;
        case 18: _t->donate(); break;
        case 19: _t->reportIssue(); break;
        case 20: _t->about(); break;
        case 21: _t->toggleFullscreen(); break;
        case 22: _t->updateUIForFullscreen(); break;
        case 23: _t->setShuffle((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->setRepeat((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: { bool _r = _t->maybeShowUpdateNag();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->stop(); break;
        case 27: _t->stateChanged((*reinterpret_cast< Media::State(*)>(_a[1]))); break;
        case 28: _t->searchFocus(); break;
        case 29: _t->tick((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 30: _t->volumeUp(); break;
        case 31: _t->volumeDown(); break;
        case 32: _t->toggleVolumeMute(); break;
        case 33: _t->volumeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 34: _t->volumeMutedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 35: _t->startFullScan((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 36: _t->fullScanFinished((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 37: _t->startIncrementalScan(); break;
        case 38: _t->incrementalScanProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 39: _t->incrementalScanFinished((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 40: _t->search((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 41: _t->suggestionAccepted((*reinterpret_cast< Suggestion*(*)>(_a[1]))); break;
        case 42: _t->searchCleared(); break;
        case 43: _t->showActionInStatusBar((*reinterpret_cast< QAction*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 44: _t->showStopAfterThisInStatusBar((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 45: _t->toggleScrobbling((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 46: _t->enableScrobbling(); break;
        case 47: _t->disableScrobbling(); break;
        case 48: _t->lastFmLogout(); break;
        case 49: _t->savePlaylist(); break;
        case 50: _t->loadPlaylist(); break;
        case 51: _t->toggleMenuVisibility(); break;
        case 52: _t->toggleMenuVisibilityWithMessage(); break;
        case 53: _t->toggleToolbarMenu(); break;
        case 54: _t->runFinetune(); break;
        case 55: _t->runFinetune((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 56: _t->runFinetune((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 43:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::currentTimeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::collectionCreated)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::currentTimeChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void MainWindow::collectionCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
