/****************************************************************************
** Meta object code from reading C++ file 'finderlistview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../musique/src/finderlistview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'finderlistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FinderListView_t {
    QByteArrayData data[16];
    char stringdata0[177];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FinderListView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FinderListView_t qt_meta_stringdata_FinderListView = {
    {
QT_MOC_LITERAL(0, 0, 14), // "FinderListView"
QT_MOC_LITERAL(1, 15, 4), // "play"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 33, 5), // "index"
QT_MOC_LITERAL(5, 39, 6), // "appear"
QT_MOC_LITERAL(6, 46, 9), // "disappear"
QT_MOC_LITERAL(7, 56, 15), // "setHoveredIndex"
QT_MOC_LITERAL(8, 72, 13), // "setHoveredRow"
QT_MOC_LITERAL(9, 86, 3), // "row"
QT_MOC_LITERAL(10, 90, 10), // "clearHover"
QT_MOC_LITERAL(11, 101, 18), // "enterPlayIconHover"
QT_MOC_LITERAL(12, 120, 17), // "exitPlayIconHover"
QT_MOC_LITERAL(13, 138, 12), // "refreshIndex"
QT_MOC_LITERAL(14, 151, 10), // "refreshRow"
QT_MOC_LITERAL(15, 162, 14) // "updatePlayIcon"

    },
    "FinderListView\0play\0\0QModelIndex\0index\0"
    "appear\0disappear\0setHoveredIndex\0"
    "setHoveredRow\0row\0clearHover\0"
    "enterPlayIconHover\0exitPlayIconHover\0"
    "refreshIndex\0refreshRow\0updatePlayIcon"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FinderListView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   72,    2, 0x0a /* Public */,
       6,    0,   73,    2, 0x0a /* Public */,
       7,    1,   74,    2, 0x0a /* Public */,
       8,    1,   77,    2, 0x0a /* Public */,
      10,    0,   80,    2, 0x0a /* Public */,
      11,    0,   81,    2, 0x0a /* Public */,
      12,    0,   82,    2, 0x0a /* Public */,
      13,    1,   83,    2, 0x0a /* Public */,
      14,    1,   86,    2, 0x0a /* Public */,
      15,    0,   89,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,

       0        // eod
};

void FinderListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FinderListView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->play((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 1: _t->appear(); break;
        case 2: _t->disappear(); break;
        case 3: _t->setHoveredIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 4: _t->setHoveredRow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->clearHover(); break;
        case 6: _t->enterPlayIconHover(); break;
        case 7: _t->exitPlayIconHover(); break;
        case 8: _t->refreshIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 9: _t->refreshRow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->updatePlayIcon(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FinderListView::*)(const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FinderListView::play)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FinderListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_meta_stringdata_FinderListView.data,
    qt_meta_data_FinderListView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FinderListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FinderListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FinderListView.stringdata0))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int FinderListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void FinderListView::play(const QModelIndex & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
