/****************************************************************************
** Meta object code from reading C++ file 'mediaview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../musique/src/mediaview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mediaview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MediaView_t {
    QByteArrayData data[22];
    char stringdata0[244];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MediaView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MediaView_t qt_meta_stringdata_MediaView = {
    {
QT_MOC_LITERAL(0, 0, 9), // "MediaView"
QT_MOC_LITERAL(1, 10, 6), // "appear"
QT_MOC_LITERAL(2, 17, 0), // ""
QT_MOC_LITERAL(3, 18, 9), // "disappear"
QT_MOC_LITERAL(4, 28, 9), // "playPause"
QT_MOC_LITERAL(5, 38, 12), // "trackRemoved"
QT_MOC_LITERAL(6, 51, 6), // "search"
QT_MOC_LITERAL(7, 58, 5), // "query"
QT_MOC_LITERAL(8, 64, 16), // "activeRowChanged"
QT_MOC_LITERAL(9, 81, 3), // "row"
QT_MOC_LITERAL(10, 85, 6), // "manual"
QT_MOC_LITERAL(11, 92, 13), // "startPlayback"
QT_MOC_LITERAL(12, 106, 12), // "stateChanged"
QT_MOC_LITERAL(13, 119, 12), // "Media::State"
QT_MOC_LITERAL(14, 132, 8), // "newState"
QT_MOC_LITERAL(15, 141, 11), // "handleError"
QT_MOC_LITERAL(16, 153, 7), // "message"
QT_MOC_LITERAL(17, 161, 16), // "playlistFinished"
QT_MOC_LITERAL(18, 178, 16), // "playbackFinished"
QT_MOC_LITERAL(19, 195, 13), // "trackFinished"
QT_MOC_LITERAL(20, 209, 13), // "aboutToFinish"
QT_MOC_LITERAL(21, 223, 20) // "currentSourceChanged"

    },
    "MediaView\0appear\0\0disappear\0playPause\0"
    "trackRemoved\0search\0query\0activeRowChanged\0"
    "row\0manual\0startPlayback\0stateChanged\0"
    "Media::State\0newState\0handleError\0"
    "message\0playlistFinished\0playbackFinished\0"
    "trackFinished\0aboutToFinish\0"
    "currentSourceChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MediaView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x0a /* Public */,
       3,    0,   80,    2, 0x0a /* Public */,
       4,    0,   81,    2, 0x0a /* Public */,
       5,    0,   82,    2, 0x0a /* Public */,
       6,    1,   83,    2, 0x0a /* Public */,
       8,    3,   86,    2, 0x08 /* Private */,
      12,    1,   93,    2, 0x08 /* Private */,
      15,    1,   96,    2, 0x08 /* Private */,
      17,    0,   99,    2, 0x08 /* Private */,
      18,    0,  100,    2, 0x08 /* Private */,
      19,    0,  101,    2, 0x08 /* Private */,
      20,    0,  102,    2, 0x08 /* Private */,
      21,    0,  103,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MediaView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MediaView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->appear(); break;
        case 1: _t->disappear(); break;
        case 2: _t->playPause(); break;
        case 3: _t->trackRemoved(); break;
        case 4: _t->search((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->activeRowChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 6: _t->stateChanged((*reinterpret_cast< Media::State(*)>(_a[1]))); break;
        case 7: _t->handleError((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 8: _t->playlistFinished(); break;
        case 9: _t->playbackFinished(); break;
        case 10: _t->trackFinished(); break;
        case 11: _t->aboutToFinish(); break;
        case 12: _t->currentSourceChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MediaView::staticMetaObject = { {
    QMetaObject::SuperData::link<View::staticMetaObject>(),
    qt_meta_stringdata_MediaView.data,
    qt_meta_data_MediaView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MediaView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MediaView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MediaView.stringdata0))
        return static_cast<void*>(this);
    return View::qt_metacast(_clname);
}

int MediaView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
