/****************************************************************************
** Meta object code from reading C++ file 'globalshortcuts.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../musique/src/globalshortcuts.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'globalshortcuts.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GlobalShortcuts_t {
    QByteArrayData data[15];
    char stringdata0[124];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GlobalShortcuts_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GlobalShortcuts_t qt_meta_stringdata_GlobalShortcuts = {
    {
QT_MOC_LITERAL(0, 0, 15), // "GlobalShortcuts"
QT_MOC_LITERAL(1, 16, 4), // "Play"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 5), // "Pause"
QT_MOC_LITERAL(4, 28, 9), // "PlayPause"
QT_MOC_LITERAL(5, 38, 4), // "Stop"
QT_MOC_LITERAL(6, 43, 9), // "StopAfter"
QT_MOC_LITERAL(7, 53, 4), // "Next"
QT_MOC_LITERAL(8, 58, 8), // "Previous"
QT_MOC_LITERAL(9, 67, 9), // "IncVolume"
QT_MOC_LITERAL(10, 77, 9), // "DecVolume"
QT_MOC_LITERAL(11, 87, 4), // "Mute"
QT_MOC_LITERAL(12, 92, 11), // "SeekForward"
QT_MOC_LITERAL(13, 104, 12), // "SeekBackward"
QT_MOC_LITERAL(14, 117, 6) // "reload"

    },
    "GlobalShortcuts\0Play\0\0Pause\0PlayPause\0"
    "Stop\0StopAfter\0Next\0Previous\0IncVolume\0"
    "DecVolume\0Mute\0SeekForward\0SeekBackward\0"
    "reload"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GlobalShortcuts[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,
       3,    0,   80,    2, 0x06 /* Public */,
       4,    0,   81,    2, 0x06 /* Public */,
       5,    0,   82,    2, 0x06 /* Public */,
       6,    0,   83,    2, 0x06 /* Public */,
       7,    0,   84,    2, 0x06 /* Public */,
       8,    0,   85,    2, 0x06 /* Public */,
       9,    0,   86,    2, 0x06 /* Public */,
      10,    0,   87,    2, 0x06 /* Public */,
      11,    0,   88,    2, 0x06 /* Public */,
      12,    0,   89,    2, 0x06 /* Public */,
      13,    0,   90,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    0,   91,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void GlobalShortcuts::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GlobalShortcuts *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->Play(); break;
        case 1: _t->Pause(); break;
        case 2: _t->PlayPause(); break;
        case 3: _t->Stop(); break;
        case 4: _t->StopAfter(); break;
        case 5: _t->Next(); break;
        case 6: _t->Previous(); break;
        case 7: _t->IncVolume(); break;
        case 8: _t->DecVolume(); break;
        case 9: _t->Mute(); break;
        case 10: _t->SeekForward(); break;
        case 11: _t->SeekBackward(); break;
        case 12: _t->reload(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::Play)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::Pause)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::PlayPause)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::Stop)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::StopAfter)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::Next)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::Previous)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::IncVolume)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::DecVolume)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::Mute)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::SeekForward)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (GlobalShortcuts::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GlobalShortcuts::SeekBackward)) {
                *result = 11;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject GlobalShortcuts::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_GlobalShortcuts.data,
    qt_meta_data_GlobalShortcuts,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GlobalShortcuts::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GlobalShortcuts::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GlobalShortcuts.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GlobalShortcuts::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void GlobalShortcuts::Play()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GlobalShortcuts::Pause()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GlobalShortcuts::PlayPause()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GlobalShortcuts::Stop()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GlobalShortcuts::StopAfter()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void GlobalShortcuts::Next()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void GlobalShortcuts::Previous()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void GlobalShortcuts::IncVolume()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void GlobalShortcuts::DecVolume()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void GlobalShortcuts::Mute()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void GlobalShortcuts::SeekForward()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void GlobalShortcuts::SeekBackward()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
